from apscheduler.schedulers.background import BackgroundScheduler
import requests

def scheduler_train_model(url='http://localhost:3000/train'):
    headers = {'Content-Type': 'application/json'}
    response = requests.put(url, headers=headers)
    if response.status_code == 200:
        print("模型训练任务调用成功")
    else:
        print("模型训练任务调用失败，状态码：", response.status_code)

# 假设你的Flask应用和训练接口运行在同一服务器上的3000端口
def scheduler(train_url='http://localhost:3000/train', hour = 4):
    # 初始化 BackgroundScheduler
    scheduler = BackgroundScheduler()
    # 添加定时任务，每天凌晨3点执行train_model函数，并传递train_url作为参数
    scheduler.add_job(scheduler_train_model, 'cron', args=[train_url], hour=hour)
    # 启动调度器
    scheduler.start()


import os
from mysql.connector import Error
from .db import fetch_data_from_mysql, check_sentences_is_unique, insert_data_into_mysql, append_data_to_file, delete_data_from_mysql

# 查询数据
def fetch_data(DB_CONFIG, query):
    try:
        return fetch_data_from_mysql(DB_CONFIG, query)
    except Error:
        raise Error("数据库查询出错")

# 插入的前置准备
def check_sentence(DB_CONFIG, query, params):
    try:
        return check_sentences_is_unique(DB_CONFIG, query, params)
    except Error:
        raise Error("数据库查询出错")

# 插入数据到数据库，并获取插入信息
def insert_data(DB_CONFIG, params, insert_query, select_query):
    try:
        return insert_data_into_mysql(DB_CONFIG, params, insert_query, select_query)
    except Error:
        raise Error("数据库插入出错")
    
# 插入到文件那种
def append_file(bieos):
    try:
        # 获取当前文件的绝对路径
        current_file_path = os.path.abspath(__file__)
        # 获取当前文件所在的目录，即nbnlp目录
        project_dir = os.path.dirname(current_file_path)
        # 构建到data目录的绝对路径
        data_dir = os.path.join(project_dir, 'data')
        # 使用绝对路径
        training_corpus = os.path.join(data_dir, 'train.tsv')
        test_corpus = os.path.join(data_dir, 'test.tsv')

        if not append_data_to_file(bieos, training_corpus) or not append_data_to_file( bieos, test_corpus):
            return False
        return True
    except Error:
        raise Error("数据库插入出错")
    
# 删除数据
def delete_data(DB_CONFIG, query, params):
    try:
        return delete_data_from_mysql(DB_CONFIG, query, params)
    except Error:
        raise Error("数据库删除出错")
    
# 训练模型
def train_model(server, params, finetune_path, transformer_path, fine_electra_small_zh_path):
    try:
        training_result = server.train_model(params, finetune_path, transformer_path, fine_electra_small_zh_path)
        if not training_result:
            return False
        return True
    except Error as e:
        raise Error("模型训练出错")



import hanlp

# 映射表
entity_type_to_description = {
    'INTEGER': '整数',
    'ORDINAL': '序数',
    'LOCATION': '地点',
    'DATE': '日期',
    'ORGANIZATION': '组织',
    'PERSON': '人名',
    'MONEY': '金钱数额',
    'DURATION': '持续时间',
    'TIME': '时间',
    'LENGTH': '长度',
    'AGE': '年龄',
    'FREQUENCY': '频率',
    'ANGLE': '角度',
    'PHONE': '电话号码',
    'PERCENT': '百分比',
    'FRACTION': '分数',
    'WEIGHT': '重量',
    'AREA': '面积',
    'CAPACITY': '容量', 
    'DECIMAL': '小数',
    'MEASURE': '测量值',
    'SPEED': '速度',
    'TEMPERATURE': '温度',
    'POSTALCODE': '邮政编码',
    'RATE': '比率',
    'WWW': '网址'
}

class HanLPWrapper:
    def __init__(self, model_path):
        """
        初始化HanLPWrapper类。

        参数:
        - model_path: 模型的路径。
        """
        self.model = hanlp.load(model_path)

    def process_text(self, text, tasks='ner'):
        """
        使用加载的HanLP模型处理文本。

        参数:
        - text: 要处理的文本。
        
        返回:
        - 处理后的结果。
        """
        result = self.model(text, tasks=tasks)
        entities_list = []
        for entity in result['ner/msra']:
            entity_text = entity[0]  
            entity_type = entity[1] 
            description = entity_type_to_description.get(entity_type, "未知类型") 
            entities_list.append({
                'text': entity_text,
                'label': entity_type,
                'description': description
            })
        return entities_list


import os
import json
from mysql.connector import Error
from flask import Flask, request, jsonify, make_response

# 默认配置
default_config = {
    'host': '0.0.0.0',
    'port': 8080,
    'debug': False,
}

# 设置环境变量
def set_environment_variable(name, value):
    """
    设置环境变量。

    参数:
    - name: 环境变量的名称。
    - value: 要设置的值。
    """
    os.environ[name] = value

# 去除空格
def trim(data, key, default=""):
    value = data.get(key, default)
    if isinstance(value, str):
        return value.strip()
    return value

# 默认为 None
def defNone(data, key):
    return data.get(key, None)

# 参数均不能为空
def haveAll(key1, key2):
    if not key1 or not key2:
        raise Error("参数均不能为空")
    
# 非空校验
def notNull(params):
    """非空校验"""
    if not params:
        raise Error("参数不能为空")
    
# 非空并且是列表
def isList(params):
    """非空并且是列表"""
    if not params or not isinstance(params, list):
        raise Error("参数必须是列表")
    
# 将数组转成 JSON 格式
def formatJson(array, key, value):
    try:
        annotations = []
        for text, label in array:
            annotation = {key: text, value: label}
            annotations.append(annotation)
        annotations_json = json.dumps(annotations)
        return annotations_json
    except Error as e:
        raise Error("数组转 JSON 格式失败")

# 预测的逻辑
def predictResult(server, text):
    try:
        result = server.predict(text)
        entities = result['ner'][0]
        return [{"key": entity[1], "value": entity[0]} for entity in entities]
    except Error:
        raise Error

# 必须是整数
def isInt(params):
    """必须是整数"""
    if not params or not isinstance(params, int):
        raise Error("参数必须是整数")

# 服务
def Service(__name__):
    return Flask(__name__)

# 解析参数
def get_request_data(params="body"):
    if params == "body":
        return request.get_json()
    else:
        return request.args

# 发送数据
def send_data(code=0, msg="数据获取成功。", data=None):
    response_content = {"code": code, "msg": msg}
    if data is not None:
        response_content["data"] = data
    status = 200 if code == 0 else code
    return make_response(jsonify(response_content), status)

# 运行服务
def run(app, config = default_config):
    final_config = {**default_config, **config}
    app.run(**final_config)



import os
import hanlp
from hanlp.components.ner.transformer_ner import TransformerNamedEntityRecognizer

class HanLPServer:
    def __init__(self):
        self.ner = None
        self.HanLP = None
        self.is_model_trained = False
        self.is_training = False

        # 获取当前文件的绝对路径
        current_file_path = os.path.abspath(__file__)
        # 获取当前文件所在的目录，即nbnlp目录
        project_dir = os.path.dirname(current_file_path)
        # 构建到data目录的绝对路径
        self.data_dir = os.path.join(project_dir, 'data')
        # 构建到model目录的绝对路径
        self.model_dir = os.path.join(project_dir, 'model')
        # 构建到offline目录的绝对路径
        # self.offline_dir = os.path.join(project_dir, 'offline')

    def train_model(self, epochs, finetune_path, transformer_path, fine_electra_small_zh_path):
        if self.is_training:
            return False

        self.is_training = True
        
        try:
            # 使用绝对路径
            your_training_corpus = os.path.join(self.data_dir, 'train.tsv')
            your_development_corpus = os.path.join(self.data_dir, 'test.tsv')

            save_dir = self.model_dir

            self.ner = TransformerNamedEntityRecognizer()

            # 使用绝对路径引用offline目录下的资源
            # finetune_path = os.path.join(self.offline_dir, 'MSRA_NER_ELECTRA_SMALL_ZH')
            # transformer_path = os.path.join(self.offline_dir, 'CHINESE_ELECTRA_TRANSFORMER')

            self.ner.fit(
                trn_data=your_training_corpus,
                dev_data=your_development_corpus,
                save_dir=save_dir,
                epochs=epochs,
                finetune=finetune_path,
                average_subwords=True,
                transformer=transformer_path,
            )

            # 使用绝对路径引用offline目录下的资源
            # fine_electra_small_zh_path = os.path.join(self.offline_dir, 'FINE_ELECTRA_SMALL_ZH')

            self.HanLP = hanlp.pipeline() \
                .append(hanlp.load(fine_electra_small_zh_path), output_key='tok') \
                .append(self.ner, output_key='ner')

            self.is_model_trained = True
            return True
        finally:
            self.is_training = False

    def predict(self, text):
        return self.HanLP([text])



from setuptools import setup, find_packages

setup(
    name='nbnlp',
    version="3.0.0",
    packages=find_packages(),
    include_package_data=True,
    author="xl",
    author_email="123456@qq.com",
    description="A short description of your package",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    package_data={
        'nbnlp': ['data/*', 'model/*'],
    },
    install_requires=[
        'mysql-connector-python',
        'hanlp',
        'Flask',
        'APScheduler',
        'requests'
    ]
)



import mysql.connector
from mysql.connector import Error
from contextlib import contextmanager

# 上下文管理器
@contextmanager
def mysql_connection(db_config):
    connection = mysql.connector.connect(**db_config)
    cursor = connection.cursor(dictionary=True)
    try:
        yield cursor
    finally:
        connection.commit()
        cursor.close()
        connection.close()

# 查询数据(sql)
def fetch_data_from_mysql(db_config, query):
    try:
        with mysql_connection(db_config) as cursor:
            cursor.execute(query)
            return cursor.fetchall()
    except Error:
        raise Error

# 插入的前置准备(sql)
def check_sentences_is_unique(db_config, query, params):
    try:
        with mysql_connection(db_config) as cursor:
            cursor.execute(query, params)
            result = cursor.fetchone()
            return result['count']
    except Error:
        raise Error
    
# 获取插入 id(sql)
def fetch_insert_id(cursor):
    try:
        # 获取插入 id
        last_insert_id_query = "SELECT LAST_INSERT_ID()"
        cursor.execute(last_insert_id_query)
        last_insert_id = cursor.fetchone()["LAST_INSERT_ID()"]
        return last_insert_id
    except Error:
        raise Error
    

# 将数据插入 MySQL 数据库并返回插入信息
def insert_data_into_mysql(db_config, params, insert_query, select_query):
    try:
        with mysql_connection(db_config) as cursor:
            # 执行插入语句
            cursor.execute(insert_query, params)

            # 获取插入 id
            last_insert_id = fetch_insert_id(cursor)
        
            # 执行查询语句
            cursor.execute(select_query, (last_insert_id,))
            record = cursor.fetchone()
            return record
    except Error:
        raise Error
    
# 将数据加入到 tsv 中
def append_data_to_file(annotations, file_path):
    try:
        with open(file_path, 'a', encoding='utf-8') as f:
            for text, label in annotations:
                f.write('\t'.join([text, label]) + '\n')
            f.write('\n')
        return True
    except IOError as e:
        return False
    
# 从数据库删除数据
def delete_data_from_mysql(db_config, delete_query, params):
    try:
        with mysql_connection(db_config) as cursor:
            # 执行删除语句
            cursor.execute(delete_query, params)
    except Error:
        raise Error
